#include "log.h"

#define _GNU_SOURCE         /* See feature_test_macros(7) */
#include <ctype.h>
#include <errno.h>
#include <pthread.h>
#include <stdarg.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <syslog.h>


/* default value during argument parsing */
static int debugLevel = LOG_INFO;
static pthread_mutex_t reportMutex;
static char *log_prefix;
static bool use_syslog;


int init_logger(const char *prefix)
{
    char *ret;
    pthread_mutex_init(&reportMutex, NULL);
    if (asprintf(&ret, "%s: ", prefix) < 0) {
        fprintf(stderr, "asprintf failed: %s\n", strerror(ENOMEM));
        return -ENOMEM;
    }
    log_prefix = ret;
    return 0;
}

void set_log_level(int level)
{
    debugLevel = level;
}

void cleanup_logger()
{
    pthread_mutex_destroy(&reportMutex);
    if (log_prefix) {
        free(log_prefix);
    }
}

void enable_syslog(void)
{
    use_syslog = true;
    openlog(log_prefix, LOG_PID, LOG_USER);
}

static void my_log3000(const char *buf)
{
    char buf2[BUFSIZ], tsbuf[30];
    const char* sp;
    struct timeval t;

    gettimeofday(&t, NULL);
    strftime(tsbuf, sizeof(tsbuf), "%Y-%m-%d %H:%M:%S", localtime(&t.tv_sec));
    (void)snprintf(buf2, BUFSIZ, "[%s.%03d] ", tsbuf, (int)t.tv_usec / 1000);

    for (sp = buf; *sp != '\0'; sp++) {
        /* TODO buffer overlow if user can control output */
        /* if hex values maps to '\n' only print it if at the end of buffer */
        if (isprint((int)*sp) || (isspace((int)*sp) && (sp[1]=='\0' || sp[2]=='\0')))
            (void)snprintf(buf2+strlen(buf2), 2, "%c", *sp);
        else
            (void)snprintf(buf2+strlen(buf2), 6, "\\%02hhx", *sp);
    }

    (void)fputs(buf2, stderr);
}

void report(int errlevel, const char *fmt, ... )
{
    if (errlevel <= debugLevel) {
        char buf[BUFSIZ];
        va_list ap;
        pthread_mutex_lock(&reportMutex);

        strncpy(buf, log_prefix, BUFSIZ);
        va_start(ap, fmt) ;
        vsnprintf(buf + strlen(buf), sizeof(buf)-strlen(buf), fmt, ap);
        va_end(ap);

        if (use_syslog) {
            syslog(errlevel, "%s", buf);
        } else {
            my_log3000(buf);
        }

        pthread_mutex_unlock(&reportMutex);
    }
}

#if 0
int main()
{
    const char buf[] = "last 'x' char will not be printed x\n";
    const char buf2[] = { 0x48, 0x61, 0x14, 0x34, 0x12, 0x45, 0x0a };
    const char buf3[] = "newline 1\n newline2\n";

    my_log3000(buf);
    my_log3000(buf2);
    my_log3000(buf3);
}
#endif
